package com.example.bdureau.bearconsole;

import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.AsyncTask;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import java.io.IOException;
import java.util.Locale;
import java.util.UUID;

public class MainScreenActivity extends AppCompatActivity {
    String address = null;
    Button btnAltiSettings, btnReadFlights, btnDisconnect,btnContinuityOnOff, btnReset;
    private ProgressDialog progress;

    BluetoothApplication myBT ;




    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //get the bluetooth Application pointer
        myBT = (BluetoothApplication) getApplication();
        //Check the local and force it if needed
        getApplicationContext().getResources().updateConfiguration(myBT.getAppLocal(), null);

        Intent newint = getIntent();
        address = newint.getStringExtra(MainActivity.EXTRA_ADDRESS); //receive the address of the bluetooth device

        setContentView(R.layout.activity_main_screen);

        btnAltiSettings = (Button)findViewById(R.id.butAltiSettings);
        btnReadFlights = (Button)findViewById(R.id.butReadFlight);
        btnDisconnect = (Button)findViewById(R.id.butDisconnect);

        btnContinuityOnOff =  (Button)findViewById(R.id.butContinuityOnOff);

        btnReset =  (Button)findViewById(R.id.butReset);


        new ConnectBT().execute(); //Call the class to connect

        //commands to be sent to bluetooth
        btnAltiSettings.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {

                Intent i= new Intent(MainScreenActivity.this, AltimeterConfigActivity.class);

                //Change the activity.
                startActivity(i);
            }
        });

        btnReadFlights.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v)
            {
                Intent i= new Intent(MainScreenActivity.this, FlightListActivity.class);
                startActivity(i);
            }
        });

        btnDisconnect.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                Disconnect(); //close connection
            }
        });

        btnReset.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v)
            {

                Intent i= new Intent(MainScreenActivity.this, ResetSettingsActivity.class);
                startActivity(i);
            }
        });

        btnContinuityOnOff.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                if (myBT.getBTConnected())
                    try {

                        // Send command to change the continuity
                        myBT.getBtSocket().getOutputStream().write("c;\n".toString().getBytes());
                        myBT.getBtSocket().getOutputStream().flush();
                    } catch (IOException e) {
                        e.printStackTrace();
                        // msg("Problem sending the command");
                    }
                msg("Continuity changed");
            }
        });

    }
    // fast way to call Toast
    private void msg(String s)
    {
        Toast.makeText(getApplicationContext(),s,Toast.LENGTH_LONG).show();
    }
    private void Disconnect()
    {
        if (myBT.getBtSocket()!=null) //If the btSocket is busy
        {
            try
            {
                myBT.getBtSocket().close(); //close connection
                myBT.setBTConnected(false);
            }
            catch (IOException e)
            { msg("Error");}
        }
        finish(); //return to the first layout

    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            Intent i= new Intent(MainScreenActivity.this, AppConfigActivity.class);
            startActivity(i);
            return true;
        }
        //open help screen
        if (id == R.id.action_help) {
            Intent i= new Intent(MainScreenActivity.this, HelpActivity.class);
            startActivity(i);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i= new Intent(MainScreenActivity.this, AboutActivity.class);
            startActivity(i);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    /* This is the connection sub class */
    private class ConnectBT extends AsyncTask<Void, Void, Void>  // UI thread
    {
        private boolean ConnectSuccess = true; //if it's here, it's almost connected

        @Override
        protected void onPreExecute()
        {
            //"Connecting...", "Please wait!!!"
            progress = ProgressDialog.show(MainScreenActivity.this,
                    getResources().getString(R.string.MS_msg1),
                    getResources().getString(R.string.MS_msg2));  //show a progress dialog
        }

        @Override
        protected Void doInBackground(Void... devices) //while the progress dialog is shown, the connection is done in background
        {

            if (myBT.getBtSocket() == null || !myBT.getBTConnected())
            {
               if( myBT.connect(address))
                   ConnectSuccess =true;
                else
                   ConnectSuccess=false;
            }
            return null;
        }
        @Override
        protected void onPostExecute(Void result) //after the doInBackground, it checks if everything went fine
        {
            super.onPostExecute(result);

            if (!ConnectSuccess)
            {
                //Connection Failed. Is it a SPP Bluetooth? Try again.
                msg(getResources().getString(R.string.MS_msg3));
                finish();
            }
            else
            {
                //Connected.
                msg(getResources().getString(R.string.MS_msg4));
                //isBtConnected = true;
                myBT.setBTConnected(true);
            }
            progress.dismiss();
        }
    }
}
